#version 330 core

in vec3 position; //vertexPosition in modelSpace
in vec3 normal; //vertex normal in modelSpace
in vec2 uv; //UV-coords


uniform vec3 lightPos_worldSpace;
uniform mat4 MVP;
uniform mat4 V;
uniform mat4 M;
//uniform mat4 normalMatrix;
uniform vec3 firePos_worldSpace;
uniform mat4 fireBiasMVP;
uniform mat4 lightBiasMVP;

out vec2 uv_varyings;
out vec3 pos_worldSpace;
out vec3 normal_camSpace;
out vec3 eyeDir_camSpace;
out vec3 lightDir_camSpace;
out vec3 fireDir_camSpace;
out vec4 fireShadowCoord;
out vec4 lightShadowCoord;


void main(){

	//vertex position in clip space
	gl_Position = MVP * vec4(position, 1.0);

	//coordinates of the vertex in the shadowMap 
	fireShadowCoord = fireBiasMVP * vec4(position, 1);

	lightShadowCoord = lightBiasMVP * vec4(position, 1);

	//position in world space
	pos_worldSpace = (M * vec4(position,1)).xyz;

	// Vector that goes from the vertex to the camera, in camera space.
	vec3 vertPos_camSpace = ( V * M * vec4(position,1)).xyz;
	eyeDir_camSpace = vec3(0,0,0) - vertPos_camSpace;

	// Vector that goes from the vertex to the light, in camera space. M is ommited because it's identity.
	vec3 lightPos_camSpace = ( V * vec4(lightPos_worldSpace,1)).xyz;
	lightDir_camSpace = lightPos_camSpace + eyeDir_camSpace;
	
	vec3 firePos_camSpace = ( V * vec4(firePos_worldSpace,1)).xyz;
	fireDir_camSpace = firePos_camSpace + eyeDir_camSpace;
	
	// Normal of the the vertex, in camera space
	normal_camSpace = ( V * M * vec4(normal,0)).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.
	
	// UV of the vertex. No special space for this one.
	uv_varyings = uv;

}